/*
 * Copyright (C) 2005-2007 Andre Noll <maan@systemlinux.org>
 *
 * Licensed under the GPL v2. For licencing details see COPYING.
 */

/** \file afh.h structures for audio format handling (para_server) */

/** \cond */
#ifdef HAVE_OGGVORBIS
#define OV_AUDIO_FORMAT " ogg"
#define OV_AUDIO_FORMAT_ARRAY , "ogg"
#else
#define OV_AUDIO_FORMAT ""
#define OV_AUDIO_FORMAT_ARRAY
#endif

#ifdef HAVE_FAAD
#define AAC_AUDIO_FORMAT " aac"
#define AAC_AUDIO_FORMAT_ARRAY , "aac"
#else
#define AAC_AUDIO_FORMAT ""
#define AAC_AUDIO_FORMAT_ARRAY
#endif

#define SUPPORTED_AUDIO_FORMATS "mp3" OV_AUDIO_FORMAT AAC_AUDIO_FORMAT
#define SUPPORTED_AUDIO_FORMATS_ARRAY "mp3" OV_AUDIO_FORMAT_ARRAY \
	AAC_AUDIO_FORMAT_ARRAY, NULL

/** \endcond */

/** size of the  audio_file info string */
#define AUDIO_FILE_INFO_SIZE 256

/**
 * Audio format dependent information. Details vary between each audio format
 * handler.
 */
struct audio_format_info {
	/** The number of chunks this audio file contains. */
	long unsigned chunks_total;
	/** The length of the audio file in seconds. */
	long unsigned seconds_total;
	/** A string that gets filled in by the audio format handler. */
	char info_string[AUDIO_FILE_INFO_SIZE];
	/**
  	 * The table that specifies the offset of the individual pieces in
  	 * the current audio file.
 	 */
	size_t *chunk_table;
	/** Period of time between sending data chunks. */
	struct timeval chunk_tv;
	/** End of file timeout - Do not load new audio file until this time. */
	struct timeval eof_tv;
	/**
	 * The header is needed by senders in case a new client connects in the
	 * middle of the stream. The length of the header defaults to zero
	 * which means that this audio format does not need any special header
	 * treatment. The audio format handler does not need to set this to
	 * zero in this case.
   	 */
	unsigned header_len;
	/**
	 * The position of the header within the audio file. Ignored if \a
	 * header_len equals zero.
	 */
	unsigned header_offset;
	/** The number of channels. */
	uint8_t channels;
	/** Frquency on Hz. */
	uint16_t frequency;
	/** Exact meaning depends on audio format. */
	uint16_t bitrate;
};

/**
 *  Structure for audio format handling.
 *
 *  There's one such struct for each supported audio format. Initially, only \a
 *  name and \a init are defined. During the startup process, para_server calls
 *  the \a init function of each audio format handler which is expected to fill
 *  in the other part of this struct.
 */
struct audio_format_handler {
	/** Name of the audio format. */
	const char *name;
	/**
	 * Pointer to the audio format handler's init function.
	 *
	 * Must initialize all function pointers and is assumed to succeed.
	 */
	void (*init)(struct audio_format_handler*);
	/** Typical file endings for files that can be handled by this afh. */
	const char **suffixes;
	/**
	 * Check if this audio format handler can handle the file.
	 *
	 * This is a  pointer to a function returning whether a given file is
	 * valid for this audio format. A negative return value indicates that
	 * this audio format handler is unable to decode the given file. On
	 * success, the function must return a positive value and fill in the
	 * given struct audio_format_info.
	 *
	 * \sa struct audio_format_info
	 */
	int (*get_file_info)(char *map, size_t numbytes,
		struct audio_format_info *afi);
};

